@@ -9,7 +9,7 @@
 int test()
 {
     const float src[] = { 0.0f, 0.0f, 0.0f, 0.0f };
-    vfloat32m1_t val = vle32_v_f32m1((const float*)(src));
+    vfloat32m1_t val = vle32_v_f32m1((const float*)(src), 4);
     return (int)vfmv_f_s_f32m1_f32(val);
 }
 #else
