@@ -5,4 +5,8 @@
 # endif
 #endif
 
+#if defined(HAVE_HPX)
+    #include <hpx/hpx_main.hpp>
+#endif
+
 CV_PERF_TEST_MAIN(core)
