@@ -40,12 +40,19 @@
 //M*/
 
 #include "precomp.hpp"
-
-#ifndef _WIN32
+#include "backend.hpp"
 
 #if defined (HAVE_GTK)
 
 #include <gtk/gtk.h>
+
+#if (GTK_MAJOR_VERSION == 3) && defined(HAVE_OPENGL)
+  #undef HAVE_OPENGL  // no support with GTK3
+#endif
+#if defined(HAVE_OPENGL) && !defined(HAVE_GTKGLEXT)
+  #undef HAVE_OPENGL  // gtkglext is required
+#endif
+
 #include <gdk/gdkkeysyms.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <stdio.h>
@@ -104,9 +111,6 @@ struct _CvImageWidgetClass
 /** Allocate new image viewer widget */
 GtkWidget*     cvImageWidgetNew      (int flags);
 
-/** Set the image to display in the widget */
-void           cvImageWidgetSetImage(CvImageWidget * widget, const CvArr *arr);
-
 // standard GTK object macros
 #define CV_IMAGE_WIDGET(obj)          G_TYPE_CHECK_INSTANCE_CAST (obj, cvImageWidget_get_type (), CvImageWidget)
 #define CV_IMAGE_WIDGET_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, cvImageWidget_get_type (), CvImageWidgetClass)
@@ -122,7 +126,10 @@ static GtkWidgetClass * parent_class = NULL;
 // flag to help size initial window
 #define CV_WINDOW_NO_IMAGE 2
 
-void cvImageWidgetSetImage(CvImageWidget * widget, const CvArr *arr){
+/** Set the image to display in the widget */
+static
+void cvImageWidgetSetImage(CvImageWidget * widget, const CvArr *arr)
+{
     CvMat * mat, stub;
     int origin=0;
 
@@ -156,6 +163,7 @@ cvImageWidgetNew (int flags)
   CvImageWidget *image_widget;
 
   image_widget = CV_IMAGE_WIDGET( gtk_widget_new (cvImageWidget_get_type (), NULL) );
+  CV_Assert(image_widget && "GTK widget creation is failed. Ensure that there is no GTK2/GTK3 libraries conflict");
   image_widget->original_image = 0;
   image_widget->scaled_image = 0;
   image_widget->flags = flags | CV_WINDOW_NO_IMAGE;
@@ -356,7 +364,7 @@ static void cvImageWidget_set_size(GtkWidget * widget, int max_width, int max_he
 
 
     }
-    assert( image_widget->scaled_image );
+    CV_Assert(image_widget->scaled_image);
 }
 
 static void
@@ -522,12 +530,13 @@ struct CvUIBase {
 
 struct CvTrackbar : CvUIBase
 {
-    CvTrackbar(const char* trackbar_name) :
+    CvTrackbar(const std::string& trackbar_name) :
         CvUIBase(CV_TRACKBAR_MAGIC_VAL),
         widget(NULL), name(trackbar_name),
         parent(NULL), data(NULL),
         pos(0), maxval(0), minval(0),
-        notify(NULL), notify2(NULL), userdata(NULL)
+        notify(NULL), notify2(NULL),  // deprecated
+        onChangeCallback(NULL), userdata(NULL)
     {
         // nothing
     }
@@ -538,20 +547,21 @@ struct CvTrackbar : CvUIBase
 
     GtkWidget* widget;
     std::string name;
-    CvWindow* parent;
+    CvWindow* parent;  // TODO weak_ptr
     int* data;
     int pos;
     int maxval;
     int minval;
-    CvTrackbarCallback notify;
-    CvTrackbarCallback2 notify2;
+    CvTrackbarCallback notify;  // deprecated
+    CvTrackbarCallback2 notify2;  // deprecated
+    TrackbarCallback onChangeCallback;
     void* userdata;
 };
 
 
 struct CvWindow : CvUIBase
 {
-    CvWindow(const char* window_name) :
+    CvWindow(const std::string& window_name) :
         CvUIBase(CV_WINDOW_MAGIC_VAL),
         widget(NULL), frame(NULL), paned(NULL), name(window_name),
         last_key(0), flags(0), status(0),
@@ -560,9 +570,10 @@ struct CvWindow : CvUIBase
         ,useGl(false), glDrawCallback(NULL), glDrawData(NULL)
 #endif
     {
-        // nothing
+        CV_LOG_INFO(NULL, "OpenCV/UI: creating GTK window: " << window_name);
     }
     ~CvWindow();
+    void destroy();
 
     GtkWidget* widget;
     GtkWidget* frame;
@@ -576,7 +587,7 @@ struct CvWindow : CvUIBase
     CvMouseCallback on_mouse;
     void* on_mouse_param;
 
-    std::vector< Ptr<CvTrackbar> > trackbars;
+    std::vector< std::shared_ptr<CvTrackbar> > trackbars;
 
 #ifdef HAVE_OPENGL
     bool useGl;
@@ -600,14 +611,14 @@ GCond*				   cond_have_key = NULL;
 GThread*			   window_thread = NULL;
 #endif
 
-static cv::Mutex& getWindowMutex()
-{
-    static cv::Mutex* g_window_mutex = new cv::Mutex();
-    return *g_window_mutex;
-}
-
 static int             last_key = -1;
-static std::vector< Ptr<CvWindow> > g_windows;
+
+static
+std::vector< std::shared_ptr<CvWindow> >& getGTKWindows()
+{
+    static std::vector< std::shared_ptr<CvWindow> > g_windows;
+    return g_windows;
+}
 
 CV_IMPL int cvInitSystem( int argc, char** argv )
 {
@@ -700,19 +711,32 @@ gpointer icvWindowThreadLoop(gpointer /*data*/)
 
 #define CV_LOCK_MUTEX() cv::AutoLock lock(getWindowMutex())
 
-static CvWindow* icvFindWindowByName( const char* name )
+static
+std::shared_ptr<CvWindow> icvFindWindowByName(const std::string& name)
 {
+    auto& g_windows = getGTKWindows();
     for(size_t i = 0; i < g_windows.size(); ++i)
     {
-        CvWindow* window = g_windows[i].get();
+        auto window = g_windows[i];
+        if (!window)
+            continue;
         if (window->name == name)
             return window;
     }
-    return NULL;
+    return std::shared_ptr<CvWindow>();
 }
 
+static inline
+std::shared_ptr<CvWindow> icvFindWindowByName(const char* name)
+{
+    CV_Assert(name);
+    return icvFindWindowByName(std::string(name));
+}
+
+
 static CvWindow* icvWindowByWidget( GtkWidget* widget )
 {
+    auto& g_windows = getGTKWindows();
     for (size_t i = 0; i < g_windows.size(); ++i)
     {
         CvWindow* window = g_windows[i].get();
@@ -722,20 +746,29 @@ static CvWindow* icvWindowByWidget( GtkWidget* widget )
     return NULL;
 }
 
+static Rect getImageRect_(const std::shared_ptr<CvWindow>& window);
+
 CvRect cvGetWindowRect_GTK(const char* name)
 {
     CV_Assert(name && "NULL name string");
 
     CV_LOCK_MUTEX();
-    CvWindow* window = icvFindWindowByName(name);
+    const auto window = icvFindWindowByName(name);
     if (!window)
         CV_Error( CV_StsNullPtr, "NULL window" );
 
+    return cvRect(getImageRect_(window));
+}
+
+static Rect getImageRect_(const std::shared_ptr<CvWindow>& window)
+{
+    CV_Assert(window);
+
     gint wx, wy;
 #ifdef HAVE_OPENGL
     if (window->useGl) {
         gtk_widget_translate_coordinates(window->widget, gtk_widget_get_toplevel(window->widget), 0, 0, &wx, &wy);
-        return cvRect(wx, wy, window->widget->allocation.width, window->widget->allocation.height);
+        return Rect(wx, wy, window->widget->allocation.width, window->widget->allocation.height);
     }
 #endif
 
@@ -743,23 +776,23 @@ CvRect cvGetWindowRect_GTK(const char* name)
     gtk_widget_translate_coordinates(&image_widget->widget, gtk_widget_get_toplevel(&image_widget->widget), 0, 0, &wx, &wy);
     if (image_widget->scaled_image) {
 #if defined (GTK_VERSION3)
-      return cvRect(wx, wy, MIN(image_widget->scaled_image->cols, gtk_widget_get_allocated_width(window->widget)),
+      return Rect(wx, wy, MIN(image_widget->scaled_image->cols, gtk_widget_get_allocated_width(window->widget)),
           MIN(image_widget->scaled_image->rows, gtk_widget_get_allocated_height(window->widget)));
 #else
-      return cvRect(wx, wy, MIN(image_widget->scaled_image->cols, window->widget->allocation.width),
+      return Rect(wx, wy, MIN(image_widget->scaled_image->cols, window->widget->allocation.width),
           MIN(image_widget->scaled_image->rows, window->widget->allocation.height));
 #endif //GTK_VERSION3
     } else if (image_widget->original_image) {
 #if defined (GTK_VERSION3)
-      return cvRect(wx, wy, MIN(image_widget->original_image->cols, gtk_widget_get_allocated_width(window->widget)),
+      return Rect(wx, wy, MIN(image_widget->original_image->cols, gtk_widget_get_allocated_width(window->widget)),
           MIN(image_widget->original_image->rows, gtk_widget_get_allocated_height(window->widget)));
 #else
-      return cvRect(wx, wy, MIN(image_widget->original_image->cols, window->widget->allocation.width),
+      return Rect(wx, wy, MIN(image_widget->original_image->cols, window->widget->allocation.width),
           MIN(image_widget->original_image->rows, window->widget->allocation.height));
 #endif //GTK_VERSION3
     }
 
-    return cvRect(-1, -1, -1, -1);
+    return Rect(-1, -1, -1, -1);
 }
 
 double cvGetModeWindow_GTK(const char* name)//YV
@@ -767,7 +800,7 @@ double cvGetModeWindow_GTK(const char* name)//YV
     CV_Assert(name && "NULL name string");
 
     CV_LOCK_MUTEX();
-    CvWindow* window = icvFindWindowByName(name);
+    const auto window = icvFindWindowByName(name);
     if (!window)
         CV_Error( CV_StsNullPtr, "NULL window" );
 
@@ -775,42 +808,52 @@ double cvGetModeWindow_GTK(const char* name)//YV
     return result;
 }
 
-
+static bool setModeWindow_(const std::shared_ptr<CvWindow>& window, int mode);
 void cvSetModeWindow_GTK( const char* name, double prop_value)//Yannick Verdie
 {
     CV_Assert(name && "NULL name string");
 
     CV_LOCK_MUTEX();
 
-    CvWindow* window = icvFindWindowByName(name);
-    if( !window )
+    const auto window = icvFindWindowByName(name);
+    if (!window)
         CV_Error( CV_StsNullPtr, "NULL window" );
 
-    if(window->flags & CV_WINDOW_AUTOSIZE)//if the flag CV_WINDOW_AUTOSIZE is set
-        return;
+    setModeWindow_(window, (int)prop_value);
+}
+
+static bool setModeWindow_(const std::shared_ptr<CvWindow>& window, int mode)
+{
+    if (window->flags & CV_WINDOW_AUTOSIZE) //if the flag CV_WINDOW_AUTOSIZE is set
+        return false;
 
     //so easy to do fullscreen here, Linux rocks !
 
-    if (window->status==CV_WINDOW_FULLSCREEN && prop_value==CV_WINDOW_NORMAL)
+    if (window->status == mode)
+        return true;
+
+    if (window->status==CV_WINDOW_FULLSCREEN && mode==CV_WINDOW_NORMAL)
     {
         gtk_window_unfullscreen(GTK_WINDOW(window->frame));
         window->status=CV_WINDOW_NORMAL;
-        return;
+        return true;
     }
 
-    if (window->status==CV_WINDOW_NORMAL && prop_value==CV_WINDOW_FULLSCREEN)
+    if (window->status==CV_WINDOW_NORMAL && mode==CV_WINDOW_FULLSCREEN)
     {
         gtk_window_fullscreen(GTK_WINDOW(window->frame));
         window->status=CV_WINDOW_FULLSCREEN;
-        return;
+        return true;
     }
+
+    return false;
 }
 
-void cv::setWindowTitle(const String& winname, const String& title)
+void setWindowTitle_GTK(const String& winname, const String& title)
 {
     CV_LOCK_MUTEX();
 
-    CvWindow* window = icvFindWindowByName(winname.c_str());
+    auto window = icvFindWindowByName(winname.c_str());
 
     if (!window)
     {
@@ -828,7 +871,7 @@ double cvGetPropWindowAutoSize_GTK(const char* name)
 
     CV_LOCK_MUTEX();
 
-    CvWindow* window = icvFindWindowByName(name);
+    const auto window = icvFindWindowByName(name);
     if (!window)
         return -1; // keep silence here
 
@@ -836,16 +879,22 @@ double cvGetPropWindowAutoSize_GTK(const char* name)
     return result;
 }
 
+static double getRatioWindow_(const std::shared_ptr<CvWindow>& window);
 double cvGetRatioWindow_GTK(const char* name)
 {
     CV_Assert(name && "NULL name string");
 
     CV_LOCK_MUTEX();
 
-    CvWindow* window = icvFindWindowByName(name);
+    const auto window = icvFindWindowByName(name);
     if (!window)
         return -1; // keep silence here
 
+    return getRatioWindow_(window);
+}
+
+static double getRatioWindow_(const std::shared_ptr<CvWindow>& window)
+{
 #if defined (GTK_VERSION3)
     double result = static_cast<double>(
         gtk_widget_get_allocated_width(window->widget)) / gtk_widget_get_allocated_height(window->widget);
@@ -862,7 +911,7 @@ double cvGetOpenGlProp_GTK(const char* name)
 
     CV_LOCK_MUTEX();
 
-    CvWindow* window = icvFindWindowByName(name);
+    const auto window = icvFindWindowByName(name);
     if (!window)
         return -1; // keep silence here
 
@@ -1048,6 +1097,7 @@ static gboolean cvImageWidget_expose(GtkWidget* widget, GdkEventExpose* event, g
 }
 #endif //GTK_VERSION3
 
+static std::shared_ptr<CvWindow> namedWindow_(const std::string& name, int flags);
 CV_IMPL int cvNamedWindow( const char* name, int flags )
 {
     cvInitSystem(name ? 1 : 0,(char**)&name);
@@ -1060,8 +1110,16 @@ CV_IMPL int cvNamedWindow( const char* name, int flags )
     {
         return 1;
     }
+    auto window = namedWindow_(name, flags);
+    return window ? 1 : 0;
+}
 
-    Ptr<CvWindow> window = makePtr<CvWindow>(name);
+static std::shared_ptr<CvWindow> namedWindow_(const std::string& name, int flags)
+{
+    cvInitSystem(0, NULL);
+
+    auto window_ptr = std::make_shared<CvWindow>(name);
+    CvWindow* window = window_ptr.get();
     window->flags = flags;
     window->status = CV_WINDOW_NORMAL;//YV
 
@@ -1116,9 +1174,12 @@ CV_IMPL int cvNamedWindow( const char* name, int flags )
 #endif //GTK_VERSION3_4
 
     gtk_widget_show( window->frame );
-    gtk_window_set_title( GTK_WINDOW(window->frame), name );
+    gtk_window_set_title(GTK_WINDOW(window->frame), name.c_str());
 
-    g_windows.push_back(window);
+    {
+        AutoLock lock(getWindowMutex());
+        getGTKWindows().push_back(window_ptr);
+    }
 
     bool b_nautosize = ((flags & CV_WINDOW_AUTOSIZE) == 0);
     gtk_window_set_resizable( GTK_WINDOW(window->frame), b_nautosize );
@@ -1134,10 +1195,10 @@ CV_IMPL int cvNamedWindow( const char* name, int flags )
 
 #ifdef HAVE_OPENGL
     if (window->useGl)
-        cvSetOpenGlContext(name);
+        cvSetOpenGlContext(name.c_str());
 #endif
 
-    return 1;
+    return window_ptr;
 }
 
 
@@ -1152,7 +1213,7 @@ CV_IMPL void cvSetOpenGlContext(const char* name)
 
     CV_LOCK_MUTEX();
 
-    CvWindow* window = icvFindWindowByName(name);
+    auto window = icvFindWindowByName(name);
     if (!window)
         CV_Error( CV_StsNullPtr, "NULL window" );
 
@@ -1172,7 +1233,7 @@ CV_IMPL void cvUpdateWindow(const char* name)
 
     CV_LOCK_MUTEX();
 
-    CvWindow* window = icvFindWindowByName(name);
+    auto window = icvFindWindowByName(name);
     if (!window)
         return;
 
@@ -1186,7 +1247,7 @@ CV_IMPL void cvSetOpenGlDrawCallback(const char* name, CvOpenGlDrawCallback call
 
     CV_LOCK_MUTEX();
 
-    CvWindow* window = icvFindWindowByName(name);
+    auto window = icvFindWindowByName(name);
     if( !window )
         return;
 
@@ -1203,13 +1264,21 @@ CV_IMPL void cvSetOpenGlDrawCallback(const char* name, CvOpenGlDrawCallback call
 
 CvWindow::~CvWindow()
 {
+    if (frame)
+        destroy();
+}
+
+inline void CvWindow::destroy()
+{
+    CV_LOG_INFO(NULL, "OpenCV/UI: destroying GTK window: " << name);
     gtk_widget_destroy(frame);
+    frame = nullptr;
 }
 
 static void checkLastWindow()
 {
     // if last window...
-    if (g_windows.empty())
+    if (getGTKWindows().empty())
     {
 #ifdef HAVE_GTHREAD
         if( thread_started )
@@ -1236,11 +1305,13 @@ static void checkLastWindow()
     }
 }
 
-static void icvDeleteWindow( CvWindow* window )
+static
+void icvDeleteWindow_( CvWindow* window )
 {
+    AutoLock lock(getWindowMutex());
+    auto& g_windows = getGTKWindows();
     bool found = false;
-    for (std::vector< Ptr<CvWindow> >::iterator i = g_windows.begin();
-         i != g_windows.end(); ++i)
+    for (auto i = g_windows.begin(); i != g_windows.end(); ++i)
     {
         if (i->get() == window)
         {
@@ -1249,8 +1320,7 @@ static void icvDeleteWindow( CvWindow* window )
             break;
         }
     }
-    CV_Assert(found && "Can't destroy non-registered window");
-
+    CV_LOG_IF_WARNING(NULL, !found, "OpenCV/GTK: Can't destroy non-registered window");
     checkLastWindow();
 }
 
@@ -1259,10 +1329,10 @@ CV_IMPL void cvDestroyWindow( const char* name )
     CV_Assert(name && "NULL name string");
 
     CV_LOCK_MUTEX();
+    auto& g_windows = getGTKWindows();
 
     bool found = false;
-    for (std::vector< Ptr<CvWindow> >::iterator i = g_windows.begin();
-         i != g_windows.end(); ++i)
+    for (auto i = g_windows.begin(); i != g_windows.end(); ++i)
     {
         if (i->get()->name == name)
         {
@@ -1271,7 +1341,7 @@ CV_IMPL void cvDestroyWindow( const char* name )
             break;
         }
     }
-    CV_Assert(found && "Can't destroy non-registered window");
+    CV_LOG_IF_ERROR(NULL, !found, "OpenCV/GTK: Can't destroy non-registered window: '" << name << "'");
 
     checkLastWindow();
 }
@@ -1282,7 +1352,7 @@ cvDestroyAllWindows( void )
 {
     CV_LOCK_MUTEX();
 
-    g_windows.clear();
+    getGTKWindows().clear();
     checkLastWindow();
 }
 
@@ -1305,7 +1375,7 @@ cvShowImage( const char* name, const CvArr* arr )
 
     CV_LOCK_MUTEX();
 
-    CvWindow* window = icvFindWindowByName(name);
+    auto window = icvFindWindowByName(name);
     if(!window)
     {
         cvNamedWindow(name, 1);
@@ -1328,16 +1398,24 @@ cvShowImage( const char* name, const CvArr* arr )
     }
 }
 
+static void resizeWindow_(const std::shared_ptr<CvWindow>& window, int width, int height);
 CV_IMPL void cvResizeWindow(const char* name, int width, int height )
 {
     CV_Assert(name && "NULL name string");
 
     CV_LOCK_MUTEX();
 
-    CvWindow* window = icvFindWindowByName(name);
+    auto window = icvFindWindowByName(name);
     if(!window)
         return;
 
+    return resizeWindow_(window, width, height);
+}
+
+static
+void resizeWindow_(const std::shared_ptr<CvWindow>& window, int width, int height)
+{
+    CV_Assert(window);
     CvImageWidget* image_widget = CV_IMAGE_WIDGET( window->widget );
     //if(image_widget->flags & CV_WINDOW_AUTOSIZE)
         //EXIT;
@@ -1357,26 +1435,30 @@ CV_IMPL void cvMoveWindow( const char* name, int x, int y )
 
     CV_LOCK_MUTEX();
 
-    CvWindow* window = icvFindWindowByName(name);
+    const auto window = icvFindWindowByName(name);
     if(!window)
         return;
 
     gtk_window_move( GTK_WINDOW(window->frame), x, y );
 }
 
-
-static CvTrackbar*
-icvFindTrackbarByName( const CvWindow* window, const char* name )
+static
+std::shared_ptr<CvTrackbar> icvFindTrackbarByName(const std::shared_ptr<CvWindow>& window, const std::string& name)
 {
-    for (size_t i = 0; i < window->trackbars.size(); ++i)
+    CV_Assert(window);
+    auto& trackbars = window->trackbars;
+    for(size_t i = 0; i < trackbars.size(); ++i)
     {
-        CvTrackbar* trackbar = window->trackbars[i].get();
+        auto trackbar = trackbars[i];
+        if (!trackbar)
+            continue;
         if (trackbar->name == name)
             return trackbar;
     }
-    return NULL;
+    return std::shared_ptr<CvTrackbar>();
 }
 
+
 static int
 icvCreateTrackbar( const char* trackbar_name, const char* window_name,
                    int* val, int count, CvTrackbarCallback on_notify,
@@ -1390,16 +1472,16 @@ icvCreateTrackbar( const char* trackbar_name, const char* window_name,
 
     CV_LOCK_MUTEX();
 
-    CvWindow* window = icvFindWindowByName(window_name);
+    const auto window = icvFindWindowByName(window_name);
     if(!window)
         return 0;
 
-    CvTrackbar* trackbar = icvFindTrackbarByName(window, trackbar_name);
-    if (!trackbar)
+    auto trackbar_ = icvFindTrackbarByName(window, trackbar_name);
+    if (!trackbar_)
     {
-        Ptr<CvTrackbar> trackbar_ = makePtr<CvTrackbar>(trackbar_name);
-        trackbar = trackbar_.get();
-        trackbar->parent = window;
+        trackbar_ = std::make_shared<CvTrackbar>(trackbar_name);
+        CvTrackbar* trackbar = trackbar_.get();
+        trackbar->parent = window.get();
         window->trackbars.push_back(trackbar_);
 
         GtkWidget* hscale_box = gtk_hbox_new( FALSE, 10 );
@@ -1418,6 +1500,8 @@ icvCreateTrackbar( const char* trackbar_name, const char* window_name,
         gtk_widget_show( hscale_box );
     }
 
+    CvTrackbar* trackbar = trackbar_.get(); CV_DbgAssert(trackbar);
+
     if( val )
     {
         int value = *val;
@@ -1444,7 +1528,6 @@ icvCreateTrackbar( const char* trackbar_name, const char* window_name,
     return 1;
 }
 
-
 CV_IMPL int
 cvCreateTrackbar( const char* trackbar_name, const char* window_name,
                   int* val, int count, CvTrackbarCallback on_notify )
@@ -1453,7 +1536,6 @@ cvCreateTrackbar( const char* trackbar_name, const char* window_name,
                              on_notify, 0, 0);
 }
 
-
 CV_IMPL int
 cvCreateTrackbar2( const char* trackbar_name, const char* window_name,
                    int* val, int count, CvTrackbarCallback2 on_notify2,
@@ -1463,6 +1545,52 @@ cvCreateTrackbar2( const char* trackbar_name, const char* window_name,
                              0, on_notify2, userdata);
 }
 
+static
+std::shared_ptr<CvTrackbar> createTrackbar_(
+    const std::shared_ptr<CvWindow>& window, const std::string& name,
+    int count,
+    TrackbarCallback onChange, void* userdata
+)
+{
+    CV_Assert(window);
+    CV_Assert(!name.empty());
+
+    if (count <= 0)
+        CV_Error(Error::StsOutOfRange, "Bad trackbar maximal value");
+
+    auto trackbar_ = std::make_shared<CvTrackbar>(name);
+    CvTrackbar* trackbar = trackbar_.get();
+    trackbar->parent = window.get();
+    window->trackbars.push_back(trackbar_);
+
+    GtkWidget* hscale_box = gtk_hbox_new( FALSE, 10 );
+    GtkWidget* hscale_label = gtk_label_new(name.c_str());
+    GtkWidget* hscale = gtk_hscale_new_with_range( 0, count, 1 );
+    gtk_scale_set_digits( GTK_SCALE(hscale), 0 );
+    //gtk_scale_set_value_pos( hscale, GTK_POS_TOP );
+    gtk_scale_set_draw_value( GTK_SCALE(hscale), TRUE );
+
+    trackbar->widget = hscale;
+    gtk_box_pack_start( GTK_BOX(hscale_box), hscale_label, FALSE, FALSE, 5 );
+    gtk_widget_show( hscale_label );
+    gtk_box_pack_start( GTK_BOX(hscale_box), hscale, TRUE, TRUE, 5 );
+    gtk_widget_show( hscale );
+    gtk_box_pack_start( GTK_BOX(window->paned), hscale_box, FALSE, FALSE, 5 );
+    gtk_widget_show( hscale_box );
+
+    trackbar->maxval = count;
+    trackbar->onChangeCallback = onChange;
+    trackbar->userdata = userdata;
+    g_signal_connect(trackbar->widget, "value-changed",
+                     G_CALLBACK(icvOnTrackbar), trackbar);
+
+    // queue a widget resize to trigger a window resize to
+    // compensate for the addition of trackbars
+    gtk_widget_queue_resize(GTK_WIDGET(window->widget));
+
+    return trackbar_;
+}
+
 
 CV_IMPL void
 cvSetMouseCallback( const char* window_name, CvMouseCallback on_mouse, void* param )
@@ -1471,7 +1599,7 @@ cvSetMouseCallback( const char* window_name, CvMouseCallback on_mouse, void* par
 
     CV_LOCK_MUTEX();
 
-    CvWindow* window = icvFindWindowByName(window_name);
+    const auto window = icvFindWindowByName(window_name);
     if (!window)
         return;
 
@@ -1487,18 +1615,18 @@ CV_IMPL int cvGetTrackbarPos( const char* trackbar_name, const char* window_name
 
     CV_LOCK_MUTEX();
 
-    CvWindow* window = icvFindWindowByName(window_name);
+    const auto window = icvFindWindowByName(window_name);
     if (!window)
         return -1;
 
-    CvTrackbar* trackbar = icvFindTrackbarByName(window,trackbar_name);
+    const auto trackbar = icvFindTrackbarByName(window,trackbar_name);
     if (!trackbar)
         return -1;
 
     return trackbar->pos;
 }
 
-
+static void setTrackbarPos_(const std::shared_ptr<CvTrackbar>& trackbar, int pos);
 CV_IMPL void cvSetTrackbarPos( const char* trackbar_name, const char* window_name, int pos )
 {
     CV_Assert(window_name && "NULL window name");
@@ -1506,24 +1634,27 @@ CV_IMPL void cvSetTrackbarPos( const char* trackbar_name, const char* window_nam
 
     CV_LOCK_MUTEX();
 
-    CvWindow* window = icvFindWindowByName(window_name);
+    const auto window = icvFindWindowByName(window_name);
     if(!window)
         return;
 
-    CvTrackbar* trackbar = icvFindTrackbarByName(window,trackbar_name);
-    if( trackbar )
-    {
-        if( pos < trackbar->minval )
-            pos = trackbar->minval;
-
-        if( pos > trackbar->maxval )
-            pos = trackbar->maxval;
-    }
-    else
+    const auto trackbar = icvFindTrackbarByName(window, trackbar_name);
+    if (!trackbar)
     {
         CV_Error( CV_StsNullPtr, "No trackbar found" );
     }
 
+    return setTrackbarPos_(trackbar, pos);
+}
+
+static void setTrackbarPos_(const std::shared_ptr<CvTrackbar>& trackbar, int pos)
+{
+    CV_Assert(trackbar);
+    CV_CheckLE(trackbar->minval, trackbar->maxval, "");
+
+    pos = std::max(pos, trackbar->minval);
+    pos = std::min(pos, trackbar->maxval);
+
     gtk_range_set_value( GTK_RANGE(trackbar->widget), pos );
 }
 
@@ -1535,11 +1666,11 @@ CV_IMPL void cvSetTrackbarMax(const char* trackbar_name, const char* window_name
 
     CV_LOCK_MUTEX();
 
-    CvWindow* window = icvFindWindowByName(window_name);
+    const auto window = icvFindWindowByName(window_name);
     if(!window)
         return;
 
-    CvTrackbar* trackbar = icvFindTrackbarByName(window,trackbar_name);
+    const auto trackbar = icvFindTrackbarByName(window,trackbar_name);
     if(!trackbar)
         return;
 
@@ -1556,11 +1687,11 @@ CV_IMPL void cvSetTrackbarMin(const char* trackbar_name, const char* window_name
 
     CV_LOCK_MUTEX();
 
-    CvWindow* window = icvFindWindowByName(window_name);
+    const auto window = icvFindWindowByName(window_name);
     if(!window)
         return;
 
-    CvTrackbar* trackbar = icvFindTrackbarByName(window,trackbar_name);
+    const auto trackbar = icvFindTrackbarByName(window,trackbar_name);
     if(!trackbar)
         return;
 
@@ -1576,7 +1707,7 @@ CV_IMPL void* cvGetWindowHandle( const char* window_name )
 
     CV_LOCK_MUTEX();
 
-    CvWindow* window = icvFindWindowByName(window_name);
+    const auto window = icvFindWindowByName(window_name);
     if(!window)
         return NULL;
 
@@ -1747,6 +1878,10 @@ static void icvOnTrackbar( GtkWidget* widget, gpointer user_data )
         trackbar->widget == widget )
     {
         trackbar->pos = pos;
+        if (trackbar->onChangeCallback)
+            trackbar->onChangeCallback(pos, trackbar->userdata);
+
+        // deprecated
         if( trackbar->data )
             *trackbar->data = pos;
         if( trackbar->notify2 )
@@ -1762,7 +1897,14 @@ static gboolean icvOnClose( GtkWidget* widget, GdkEvent* /*event*/, gpointer use
     if( window->signature == CV_WINDOW_MAGIC_VAL &&
         window->frame == widget )
     {
-        icvDeleteWindow(window);
+        try
+        {
+            icvDeleteWindow_(window);
+        }
+        catch (...)
+        {
+            CV_LOG_WARNING(NULL, "OpenCV/GTK: unexpected C++ exception in icvDeleteWindow_");
+        }
     }
     return TRUE;
 }
@@ -1771,24 +1913,26 @@ static gboolean icvOnClose( GtkWidget* widget, GdkEvent* /*event*/, gpointer use
 static gboolean icvOnMouse( GtkWidget *widget, GdkEvent *event, gpointer user_data )
 {
     // TODO move this logic to CvImageWidget
+    // TODO add try-catch wrappers into all callbacks
     CvWindow* window = (CvWindow*)user_data;
+    if (!window || !widget ||
+        window->signature != CV_WINDOW_MAGIC_VAL ||
+        window->widget != widget ||
+        !window->on_mouse)
+        return FALSE;
+
     CvPoint2D32f pt32f = {-1., -1.};
     CvPoint pt = {-1,-1};
     int cv_event = -1, state = 0, flags = 0;
     CvImageWidget * image_widget = CV_IMAGE_WIDGET( widget );
 
-    if( window->signature != CV_WINDOW_MAGIC_VAL ||
-        window->widget != widget || !window->widget ||
-        !window->on_mouse /*|| !image_widget->original_image*/)
-        return FALSE;
-
     if( event->type == GDK_MOTION_NOTIFY )
     {
         GdkEventMotion* event_motion = (GdkEventMotion*)event;
 
         cv_event = CV_EVENT_MOUSEMOVE;
-        pt32f.x = cvRound(event_motion->x);
-        pt32f.y = cvRound(event_motion->y);
+        pt32f.x = cvFloor(event_motion->x);
+        pt32f.y = cvFloor(event_motion->y);
         state = event_motion->state;
     }
     else if( event->type == GDK_BUTTON_PRESS ||
@@ -1796,8 +1940,8 @@ static gboolean icvOnMouse( GtkWidget *widget, GdkEvent *event, gpointer user_da
              event->type == GDK_2BUTTON_PRESS )
     {
         GdkEventButton* event_button = (GdkEventButton*)event;
-        pt32f.x = cvRound(event_button->x);
-        pt32f.y = cvRound(event_button->y);
+        pt32f.x = cvFloor(event_button->x);
+        pt32f.y = cvFloor(event_button->y);
 
 
         if( event_button->type == GDK_BUTTON_PRESS )
@@ -1874,9 +2018,12 @@ static gboolean icvOnMouse( GtkWidget *widget, GdkEvent *event, gpointer user_da
             pt = cvPointFrom32f( pt32f );
         }
 
-//        if((unsigned)pt.x < (unsigned)(image_widget->original_image->width) &&
-//           (unsigned)pt.y < (unsigned)(image_widget->original_image->height) )
+        if (!image_widget->original_image/*OpenGL*/ || (
+               (unsigned)pt.x < (unsigned)(image_widget->original_image->width) &&
+               (unsigned)pt.y < (unsigned)(image_widget->original_image->height)
+            ))
         {
+            state &= gtk_accelerator_get_default_mod_mask();
             flags |= BIT_MAP(state, GDK_SHIFT_MASK,   CV_EVENT_FLAG_SHIFTKEY) |
                 BIT_MAP(state, GDK_CONTROL_MASK, CV_EVENT_FLAG_CTRLKEY)  |
                 BIT_MAP(state, GDK_MOD1_MASK,    CV_EVENT_FLAG_ALTKEY)   |
@@ -1916,7 +2063,7 @@ CV_IMPL int cvWaitKey( int delay )
             expired = !g_cond_timed_wait(cond_have_key, last_key_mutex, &timer);
         }
         else{
-            if (g_windows.empty())
+            if (getGTKWindows().empty())
             {
                 CV_LOG_WARNING(NULL, "cv::waitKey() is called without timeout and missing active windows. Ignoring");
             }
@@ -1928,7 +2075,8 @@ CV_IMPL int cvWaitKey( int delay )
         }
         my_last_key = last_key;
         g_mutex_unlock(last_key_mutex);
-        if(expired || g_windows.empty()){
+        if (expired || getGTKWindows().empty())
+        {
             return -1;
         }
         return my_last_key;
@@ -1941,7 +2089,7 @@ CV_IMPL int cvWaitKey( int delay )
         if( delay > 0 )
             timer = g_timeout_add( delay, icvAlarm, &expired );
         last_key = -1;
-        while( gtk_main_iteration_do(TRUE) && last_key < 0 && !expired && (delay > 0 || !g_windows.empty()))
+        while( gtk_main_iteration_do(TRUE) && last_key < 0 && !expired && (delay > 0 || !getGTKWindows().empty()))
             ;
 
         if( delay > 0 && !expired )
@@ -1950,8 +2098,340 @@ CV_IMPL int cvWaitKey( int delay )
     return last_key;
 }
 
+namespace cv { namespace impl {
+
+using namespace cv::highgui_backend;
+
+class GTKTrackbar;
+
+class GTKWindow
+        : public UIWindow
+        , public std::enable_shared_from_this<GTKWindow>
+{
+protected:
+    const std::string name_;
+    std::weak_ptr<CvWindow> window_;
+    std::map<std::string, std::shared_ptr<GTKTrackbar> > trackbars_;
+public:
+    GTKWindow(const std::string& name, const std::shared_ptr<CvWindow>& window)
+        : name_(name)
+        , window_(window)
+    {
+        // nothing
+    }
+
+    ~GTKWindow() CV_OVERRIDE
+    {
+        if (!window_.expired())
+            destroy();
+        CV_LOG_DEBUG(NULL, "OpenCV/UI/GTK: GTKWindow(" << name_ << ") is disposed");
+    }
+
+    const std::string& getID() const CV_OVERRIDE { return name_; }
+
+    bool isActive() const CV_OVERRIDE { return !window_.expired(); }
+
+    void destroy() CV_OVERRIDE
+    {
+        cv::AutoLock lock(getWindowMutex());
+        if (!window_.expired())
+        {
+            auto window = window_.lock();
+            if (window)
+                window->destroy();
+            window_.reset();
+        }
+    }
+
+    void imshow(InputArray image) CV_OVERRIDE
+    {
+        auto window = window_.lock();
+        CV_Assert(window);
+        CvImageWidget* image_widget = CV_IMAGE_WIDGET(window->widget);
+        CV_Assert(image_widget);
+        Mat img = image.getMat();
+        CvMat c_img = cvMat(img);  // TODO Drop C-API
+        cvImageWidgetSetImage(image_widget, &c_img);
+    }
+
+    double getProperty(int prop) const CV_OVERRIDE
+    {
+        auto window = window_.lock();
+        CV_Assert(window);
+        // see cvGetWindowProperty
+        switch (prop)
+        {
+        case CV_WND_PROP_FULLSCREEN:
+            return (double)window->status;
+
+        case CV_WND_PROP_AUTOSIZE:
+            return (window->flags & CV_WINDOW_AUTOSIZE) ? 1.0 : 0.0;
+
+        case CV_WND_PROP_ASPECTRATIO:
+            return getRatioWindow_(window);
+
+#ifdef HAVE_OPENGL
+        case CV_WND_PROP_OPENGL:
+            return window->useGl ? 1.0 : 0.0;
+#endif
+
+        default:
+            break;
+        }
+        return std::numeric_limits<double>::quiet_NaN();
+    }
+
+    bool setProperty(int prop, double value) CV_OVERRIDE
+    {
+        auto window = window_.lock();
+        CV_Assert(window);
+        // see cvSetWindowProperty
+        switch (prop)
+        {
+        case CV_WND_PROP_FULLSCREEN:
+            if (value != CV_WINDOW_NORMAL && value != CV_WINDOW_FULLSCREEN)  // bad arg
+                break;
+            setModeWindow_(window, value);
+            return true;
+
+        default:
+            break;
+        }
+        return false;
+    }
+
+    void resize(int width, int height) CV_OVERRIDE
+    {
+        auto window = window_.lock();
+        CV_Assert(window);
+        resizeWindow_(window, width, height);
+    }
+
+    void move(int x, int y) CV_OVERRIDE
+    {
+        auto window = window_.lock();
+        CV_Assert(window);
+        gtk_window_move(GTK_WINDOW(window->frame), x, y);
+    }
+
+    Rect getImageRect() const CV_OVERRIDE
+    {
+        auto window = window_.lock();
+        CV_Assert(window);
+        return getImageRect_(window);
+    }
+
+    void setTitle(const std::string& title) CV_OVERRIDE
+    {
+        auto window = window_.lock();
+        CV_Assert(window);
+        gtk_window_set_title(GTK_WINDOW(window->frame), title.c_str());
+    }
+
+    void setMouseCallback(MouseCallback onMouse, void* userdata /*= 0*/) CV_OVERRIDE
+    {
+        auto window = window_.lock();
+        CV_Assert(window);
+        window->on_mouse = onMouse;
+        window->on_mouse_param = userdata;
+    }
+
+    std::shared_ptr<UITrackbar> createTrackbar(
+        const std::string& name,
+        int count,
+        TrackbarCallback onChange /*= 0*/,
+        void* userdata /*= 0*/
+    ) CV_OVERRIDE
+    {
+        auto window = window_.lock();
+        CV_Assert(window);
+        CV_LOG_INFO(NULL, "OpenCV/UI: Creating GTK trackbar at '" << name_ << "': '" << name << "'");
+        auto trackbar = createTrackbar_(window, name, count, onChange, userdata);
+        auto ui_trackbar = std::make_shared<GTKTrackbar>(name, trackbar, shared_from_this());
+        {
+            cv::AutoLock lock(getWindowMutex());
+            trackbars_.emplace(name, ui_trackbar);
+        }
+        return std::static_pointer_cast<UITrackbar>(ui_trackbar);
+    }
+
+    std::shared_ptr<UITrackbar> findTrackbar(const std::string& name) CV_OVERRIDE
+    {
+        cv::AutoLock lock(getWindowMutex());
+        auto i = trackbars_.find(name);
+        if (i != trackbars_.end())
+        {
+            return std::static_pointer_cast<UITrackbar>(i->second);
+        }
+        return std::shared_ptr<UITrackbar>();
+    }
+};  // GTKWindow
+
+
+class GTKTrackbar : public UITrackbar
+{
+protected:
+    /*const*/ std::string name_;
+    std::weak_ptr<CvTrackbar> trackbar_;
+    std::weak_ptr<GTKWindow> parent_;
+    std::map<std::string, std::shared_ptr<GTKTrackbar> > trackbars_;
+public:
+    GTKTrackbar(const std::string& name, const std::shared_ptr<CvTrackbar>& trackbar, const std::shared_ptr<GTKWindow>& parent)
+        : trackbar_(trackbar)
+        , parent_(parent)
+    {
+        name_ = std::string("<") + name + ">@" + parent->getID();
+    }
+
+    ~GTKTrackbar() CV_OVERRIDE
+    {
+        if (!trackbar_.expired())
+            destroy();
+        CV_LOG_DEBUG(NULL, "OpenCV/UI/GTK: GTKTrackbar(" << name_ << ") is disposed");
+    }
+
+    const std::string& getID() const CV_OVERRIDE { return name_; }
+
+    bool isActive() const CV_OVERRIDE { return !trackbar_.expired(); }
+
+    void destroy() CV_OVERRIDE
+    {
+        // nothing (destroyed with parent window, dedicated trackbar removal is not supported)
+    }
+
+    int getPos() const CV_OVERRIDE
+    {
+        auto trackbar = trackbar_.lock();
+        CV_Assert(trackbar);
+        return trackbar->pos;
+    }
+    void setPos(int pos) CV_OVERRIDE
+    {
+        auto trackbar = trackbar_.lock();
+        CV_Assert(trackbar);
+        return setTrackbarPos_(trackbar, pos);
+    }
+
+    cv::Range getRange() const CV_OVERRIDE
+    {
+        auto trackbar = trackbar_.lock();
+        CV_Assert(trackbar);
+        return cv::Range(trackbar->minval, trackbar->maxval);
+    }
+
+    void setRange(const cv::Range& range) CV_OVERRIDE
+    {
+        auto trackbar = trackbar_.lock();
+        CV_Assert(trackbar);
+        CV_CheckLE(range.start, range.end, "Invalid trackbar range");
+        gtk_range_set_range(GTK_RANGE(trackbar->widget), range.start, range.end);
+    }
+};  // GTKTrackbar
+
+
+class GTKBackendUI : public UIBackend
+{
+public:
+    GTKBackendUI()
+    {
+        // NB: avoid static initialization order fiasco
+        (void)getGTKWindows();
+    }
+    ~GTKBackendUI() CV_OVERRIDE
+    {
+        destroyAllWindows();
+    }
+
+    void destroyAllWindows() CV_OVERRIDE
+    {
+        cvDestroyAllWindows();
+    }
+
+    // namedWindow
+    virtual std::shared_ptr<UIWindow> createWindow(
+        const std::string& winname,
+        int flags
+    ) CV_OVERRIDE
+    {
+        CV_LOG_INFO(NULL, "OpenCV/UI: Creating GTK window: " << winname << " (" << flags << ")");
+        auto window = namedWindow_(winname, flags);
+        auto ui_window = std::make_shared<GTKWindow>(winname, window);
+        return ui_window;
+    }
+
+    int waitKeyEx(int delay) CV_OVERRIDE
+    {
+        return cvWaitKey(delay);
+    }
+    int pollKey() CV_OVERRIDE
+    {
+        return cvWaitKey(1);  // TODO
+    }
+};  // GTKBackendUI
+
+static
+std::shared_ptr<GTKBackendUI>& getInstance()
+{
+    static std::shared_ptr<GTKBackendUI> g_instance = std::make_shared<GTKBackendUI>();
+    return g_instance;
+}
+
+} // namespace impl
+
+#ifndef BUILD_PLUGIN
+namespace highgui_backend {
+
+std::shared_ptr<UIBackend> createUIBackendGTK()
+{
+    return impl::getInstance();
+}
+
+}  // namespace highgui_backend
+#endif
+
+}  // namespace
+
+#ifdef BUILD_PLUGIN
+
+#define ABI_VERSION 0
+#define API_VERSION 0
+#include "plugin_api.hpp"
+
+static
+CvResult cv_getInstance(CV_OUT CvPluginUIBackend* handle) CV_NOEXCEPT
+{
+    try
+    {
+        if (!handle)
+            return CV_ERROR_FAIL;
+        *handle = cv::impl::getInstance().get();
+        return CV_ERROR_OK;
+    }
+    catch (...)
+    {
+        return CV_ERROR_FAIL;
+    }
+}
+
+static const OpenCV_UI_Plugin_API plugin_api =
+{
+    {
+        sizeof(OpenCV_UI_Plugin_API), ABI_VERSION, API_VERSION,
+        CV_VERSION_MAJOR, CV_VERSION_MINOR, CV_VERSION_REVISION, CV_VERSION_STATUS,
+        "GTK" CVAUX_STR(GTK_MAJOR_VERSION) " OpenCV UI plugin"
+    },
+    {
+        /*  1*/cv_getInstance
+    }
+};
+
+const OpenCV_UI_Plugin_API* CV_API_CALL opencv_ui_plugin_init_v0(int requested_abi_version, int requested_api_version, void* /*reserved=NULL*/) CV_NOEXCEPT
+{
+    if (requested_abi_version == ABI_VERSION && requested_api_version <= API_VERSION)
+        return &plugin_api;
+    return NULL;
+}
+
+#endif  // BUILD_PLUGIN
 
 #endif  // HAVE_GTK
-#endif  // _WIN32
-
-/* End of file. */
