@@ -78,7 +78,7 @@ int  gdalPaletteInterpretation2OpenCV( GDALPaletteInterp const& paletteInterp, G
 
         /// RGB
         case GPI_RGB:
-            if( gdalType == GDT_Byte    ){ return CV_8UC1;  }
+            if( gdalType == GDT_Byte    ){ return CV_8UC3;  }
             if( gdalType == GDT_UInt16  ){ return CV_16UC3; }
             if( gdalType == GDT_Int16   ){ return CV_16SC3; }
             if( gdalType == GDT_UInt32  ){ return CV_32SC3; }
@@ -104,58 +104,32 @@ int gdal2opencv( const GDALDataType& gdalType, const int& channels ){
 
         /// UInt8
         case GDT_Byte:
-            if( channels == 1 ){ return CV_8UC1; }
-            if( channels == 3 ){ return CV_8UC3; }
-            if( channels == 4 ){ return CV_8UC4; }
-            else { return CV_8UC(channels); }
-            return -1;
+            return CV_8UC(channels);
 
         /// UInt16
         case GDT_UInt16:
-            if( channels == 1 ){ return CV_16UC1; }
-            if( channels == 3 ){ return CV_16UC3; }
-            if( channels == 4 ){ return CV_16UC4; }
-            else { return CV_16UC(channels); }
-            return -1;
+            return CV_16UC(channels);
 
         /// Int16
         case GDT_Int16:
-            if( channels == 1 ){ return CV_16SC1; }
-            if( channels == 3 ){ return CV_16SC3; }
-            if( channels == 4 ){ return CV_16SC4; }
-            else { return CV_16SC(channels); }
-            return -1;
+            return CV_16SC(channels);
 
         /// UInt32
         case GDT_UInt32:
         case GDT_Int32:
-            if( channels == 1 ){ return CV_32SC1; }
-            if( channels == 3 ){ return CV_32SC3; }
-            if( channels == 4 ){ return CV_32SC4; }
-            else { return CV_32SC(channels); }
-            return -1;
+            return CV_32SC(channels);
 
         case GDT_Float32:
-            if( channels == 1 ){ return CV_32FC1; }
-            if( channels == 3 ){ return CV_32FC3; }
-            if( channels == 4 ){ return CV_32FC4; }
-            else { return CV_32FC(channels); }
-            return -1;
+            return CV_32FC(channels);
 
         case GDT_Float64:
-            if( channels == 1 ){ return CV_64FC1; }
-            if( channels == 3 ){ return CV_64FC3; }
-            if( channels == 4 ){ return CV_64FC4; }
-            else { return CV_64FC(channels); }
-            return -1;
+            return CV_64FC(channels);
 
         default:
             std::cout << "Unknown GDAL Data Type" << std::endl;
             std::cout << "Type: " << GDALGetDataTypeName(gdalType) << std::endl;
             return -1;
     }
-
-    return -1;
 }
 
 /**
@@ -163,7 +137,6 @@ int gdal2opencv( const GDALDataType& gdalType, const int& channels ){
 */
 GdalDecoder::GdalDecoder(){
 
-
     // set a dummy signature
     m_signature="0";
     for( size_t i=0; i<160; i++ ){
@@ -182,7 +155,6 @@ GdalDecoder::GdalDecoder(){
 */
 GdalDecoder::~GdalDecoder(){
 
-
     if( m_dataset != NULL ){
        close();
     }
@@ -284,30 +256,35 @@ void write_pixel( const double& pixelValue,
 
     // input: 3 channel, output: 3 channel
     else if( gdalChannels == 3 && image.channels() == 3 ){
-        if( image.depth() == CV_8U ){  image.at<Vec3b>(row,col)[channel] = newValue;  }
-        else if( image.depth() == CV_16U ){  image.ptr<Vec3s>(row,col)[channel] = newValue;  }
-        else if( image.depth() == CV_16S ){  image.ptr<Vec3s>(row,col)[channel] = newValue;  }
-        else if( image.depth() == CV_32S ){  image.ptr<Vec3i>(row,col)[channel] = newValue;  }
-        else if( image.depth() == CV_32F ){  image.ptr<Vec3f>(row,col)[channel] = newValue;  }
-        else if( image.depth() == CV_64F ){  image.ptr<Vec3d>(row,col)[channel] = newValue;  }
+        if( image.depth() == CV_8U ){  (*image.ptr<Vec3b>(row,col))[channel] = newValue;  }
+        else if( image.depth() == CV_16U ){  (*image.ptr<Vec3s>(row,col))[channel] = newValue;  }
+        else if( image.depth() == CV_16S ){  (*image.ptr<Vec3s>(row,col))[channel] = newValue;  }
+        else if( image.depth() == CV_32S ){  (*image.ptr<Vec3i>(row,col))[channel] = newValue;  }
+        else if( image.depth() == CV_32F ){  (*image.ptr<Vec3f>(row,col))[channel] = newValue;  }
+        else if( image.depth() == CV_64F ){  (*image.ptr<Vec3d>(row,col))[channel] = newValue;  }
         else{ throw std::runtime_error("Unknown image depth, gdal: 3, image: 3"); }
     }
 
     // input: 4 channel, output: 3 channel
     else if( gdalChannels == 4 && image.channels() == 3 ){
         if( channel >= 4 ){ return; }
-        else if( image.depth() == CV_8U  && channel < 4 ){  image.ptr<Vec3b>(row,col)[channel] = newValue;  }
-        else if( image.depth() == CV_16U && channel < 4 ){  image.ptr<Vec3s>(row,col)[channel] = newValue;  }
-        else if( image.depth() == CV_16S && channel < 4 ){  image.ptr<Vec3s>(row,col)[channel] = newValue;  }
-        else if( image.depth() == CV_32S && channel < 4 ){  image.ptr<Vec3i>(row,col)[channel] = newValue;  }
-        else if( image.depth() == CV_32F && channel < 4 ){  image.ptr<Vec3f>(row,col)[channel] = newValue;  }
-        else if( image.depth() == CV_64F && channel < 4 ){  image.ptr<Vec3d>(row,col)[channel] = newValue;  }
+        else if( image.depth() == CV_8U  && channel < 4 ){  (*image.ptr<Vec3b>(row,col))[channel] = newValue;  }
+        else if( image.depth() == CV_16U && channel < 4 ){  (*image.ptr<Vec3s>(row,col))[channel] = newValue;  }
+        else if( image.depth() == CV_16S && channel < 4 ){  (*image.ptr<Vec3s>(row,col))[channel] = newValue;  }
+        else if( image.depth() == CV_32S && channel < 4 ){  (*image.ptr<Vec3i>(row,col))[channel] = newValue;  }
+        else if( image.depth() == CV_32F && channel < 4 ){  (*image.ptr<Vec3f>(row,col))[channel] = newValue;  }
+        else if( image.depth() == CV_64F && channel < 4 ){  (*image.ptr<Vec3d>(row,col))[channel] = newValue;  }
         else{ throw std::runtime_error("Unknown image depth, gdal: 4, image: 3"); }
     }
 
     // input: 4 channel, output: 4 channel
     else if( gdalChannels == 4 && image.channels() == 4 ){
-        if( image.depth() == CV_8U ){  image.at<Vec4b>(row,col)[channel] = newValue;  }
+        if( image.depth() == CV_8U ){  (*image.ptr<Vec4b>(row,col))[channel] = newValue;  }
+        else if( image.depth() == CV_16U ){  (*image.ptr<Vec4s>(row,col))[channel] = newValue;  }
+        else if( image.depth() == CV_16S ){  (*image.ptr<Vec4s>(row,col))[channel] = newValue;  }
+        else if( image.depth() == CV_32S ){  (*image.ptr<Vec4i>(row,col))[channel] = newValue;  }
+        else if( image.depth() == CV_32F ){  (*image.ptr<Vec4f>(row,col))[channel] = newValue;  }
+        else if( image.depth() == CV_64F ){  (*image.ptr<Vec4d>(row,col))[channel] = newValue;  }
         else{ throw std::runtime_error("Unknown image depth, gdal: 4, image: 4"); }
     }
 
@@ -378,10 +355,7 @@ bool GdalDecoder::readData( Mat& img ){
 
 
     // make sure the image is the proper size
-    if( img.size().height != m_height ){
-        return false;
-    }
-    if( img.size().width != m_width ){
+    if( img.size() != Size(m_width, m_height) ){
         return false;
     }
 
@@ -393,7 +367,6 @@ bool GdalDecoder::readData( Mat& img ){
     // set the image to zero
     img = 0;
 
-
     // iterate over each raster band
     // note that OpenCV does bgr rather than rgb
     int nChannels = m_dataset->GetRasterCount();
@@ -415,6 +388,30 @@ bool GdalDecoder::readData( Mat& img ){
         // get the GDAL Band
         GDALRasterBand* band = m_dataset->GetRasterBand(c+1);
 
+        /* Map palette band and gray band to color index 0 and red, green,
+           blue, alpha bands to BGRA indexes. Note: ignoring HSL, CMY,
+           CMYK, and YCbCr color spaces, rather than converting them
+           to BGR. */
+        int color = 0;
+        switch (band->GetColorInterpretation()) {
+        case GCI_PaletteIndex:
+        case GCI_GrayIndex:
+        case GCI_BlueBand:
+            color = 0;
+            break;
+        case GCI_GreenBand:
+            color = 1;
+            break;
+        case GCI_RedBand:
+            color = 2;
+            break;
+        case GCI_AlphaBand:
+            color = 3;
+            break;
+        default:
+            CV_Error(cv::Error::StsError, "Invalid/unsupported mode");
+        }
+
         // make sure the image band has the same dimensions as the image
         if( band->GetXSize() != m_width || band->GetYSize() != m_height ){ return false; }
 
@@ -429,7 +426,8 @@ bool GdalDecoder::readData( Mat& img ){
         for( int y=0; y<nRows; y++ ){
 
             // get the entire row
-            band->RasterIO( GF_Read, 0, y, nCols, 1, scanline, nCols, 1, GDT_Float64, 0, 0);
+            CPLErr err = band->RasterIO( GF_Read, 0, y, nCols, 1, scanline, nCols, 1, GDT_Float64, 0, 0);
+            CV_Assert(err == CE_None);
 
             // set inside the image
             for( int x=0; x<nCols; x++ ){
@@ -437,18 +435,16 @@ bool GdalDecoder::readData( Mat& img ){
                 // set depending on image types
                 //   given boost, I would use enable_if to speed up.  Avoid for now.
                 if( hasColorTable == false ){
-                    write_pixel( scanline[x], gdalType, nChannels, img, y, x, c );
+                    write_pixel( scanline[x], gdalType, nChannels, img, y, x, color );
                 }
                 else{
-                    write_ctable_pixel( scanline[x], gdalType, gdalColorTable, img, y, x, c );
+                    write_ctable_pixel( scanline[x], gdalType, gdalColorTable, img, y, x, color );
                 }
             }
         }
 
         // delete our temp pointer
         delete [] scanline;
-
-
     }
 
     return true;
@@ -473,7 +469,7 @@ bool GdalDecoder::readHeader(){
         return false;
     }
 
-    //extract the driver infomation
+    //extract the driver information
     m_driver = m_dataset->GetDriver();
 
     // if the driver failed, then exit
@@ -557,15 +553,14 @@ ImageDecoder GdalDecoder::newDecoder()const{
 */
 bool GdalDecoder::checkSignature( const String& signature )const{
 
-
     // look for NITF
-    std::string str = signature.c_str();
+    std::string str(signature);
     if( str.substr(0,4).find("NITF") != std::string::npos ){
         return true;
     }
 
     // look for DTED
-    if( str.substr(140,4) == "DTED" ){
+    if( str.size() > 144 && str.substr(140,4) == "DTED" ){
         return true;
     }
 
@@ -574,4 +569,4 @@ bool GdalDecoder::checkSignature( const String& signature )const{
 
 } /// End of cv Namespace
 
-#endif /**< End  of HAVE_GDAL Definition */
\ No newline at end of file
+#endif /**< End  of HAVE_GDAL Definition */
