@@ -1,25 +1,21 @@
+// This file is part of OpenCV project.
+// It is subject to the license terms in the LICENSE file found in the top-level directory
+// of this distribution and at http://opencv.org/license.html
+
 #include "common.h"
 
 #include "opencv2/opencv_modules.hpp"
 
-#ifdef HAVE_OPENCV_NONFREE
-#  include "opencv2/nonfree/nonfree.hpp"
-#endif
-
 #ifdef HAVE_OPENCV_FEATURES2D
-#  include "opencv2/features2d/features2d.hpp"
+#  include "opencv2/features2d.hpp"
 #endif
 
 #ifdef HAVE_OPENCV_VIDEO
-#  include "opencv2/video/video.hpp"
-#endif
-
-#ifdef HAVE_OPENCV_ML
-#  include "opencv2/ml/ml.hpp"
+#  include "opencv2/video.hpp"
 #endif
 
 #ifdef HAVE_OPENCV_CONTRIB
-#  include "opencv2/contrib/contrib.hpp"
+#  include "opencv2/contrib.hpp"
 #endif
 
 extern "C" {
@@ -31,26 +27,6 @@ JNI_OnLoad(JavaVM* vm, void* )
     if (vm->GetEnv((void**) &env, JNI_VERSION_1_6) != JNI_OK)
         return -1;
 
-    bool init = true;
-#ifdef HAVE_OPENCV_NONFREE
-    init &= cv::initModule_nonfree();
-#endif
-#ifdef HAVE_OPENCV_FEATURES2D
-    init &= cv::initModule_features2d();
-#endif
-#ifdef HAVE_OPENCV_VIDEO
-    init &= cv::initModule_video();
-#endif
-#ifdef HAVE_OPENCV_ML
-    init &= cv::initModule_ml();
-#endif
-    #ifdef HAVE_OPENCV_CONTRIB
-    init &= cv::initModule_contrib();
-#endif
-
-    if(!init)
-        return -1;
-
     /* get class with (*env)->FindClass */
     /* register methods with (*env)->RegisterNatives */
 
