@@ -104,6 +104,10 @@ def makeWhiteList(module_list):
     return wl
 
 white_list = None
+namespace_prefix_override = {
+    'dnn' : '',
+    'aruco' : '',
+}
 
 # Features to be exported
 export_enums = False
@@ -119,6 +123,7 @@ type_dict = {
     'InputOutputArray': 'cv::Mat&',
     'InputArrayOfArrays': 'const std::vector<cv::Mat>&',
     'OutputArrayOfArrays': 'std::vector<cv::Mat>&',
+    'string': 'std::string',
     'String': 'std::string',
     'const String&':'const std::string&'
 }
@@ -270,6 +275,8 @@ class FuncVariant(object):
 
 class FuncInfo(object):
     def __init__(self, class_name, name, cname, namespace, isconstructor):
+        self.name_id = '_'.join([namespace] + ([class_name] if class_name else []) + [name])  # unique id for dict key
+
         self.class_name = class_name
         self.name = name
         self.cname = cname
@@ -294,9 +301,9 @@ class JSWrapperGenerator(object):
         self.bindings = []
         self.wrapper_funcs = []
 
-        self.classes = {}
+        self.classes = {}  # FIXIT 'classes' should belong to 'namespaces'
         self.namespaces = {}
-        self.enums = {}
+        self.enums = {}  # FIXIT 'enums' should belong to 'namespaces'
 
         self.parser = hdr_parser.CppHeaderParser()
         self.class_idx = 0
@@ -418,7 +425,8 @@ class JSWrapperGenerator(object):
         else:
             func_map = self.namespaces.setdefault(namespace, Namespace()).funcs
 
-        func = func_map.setdefault(name, FuncInfo(class_name, name, cpp_name, namespace, is_constructor))
+        fi = FuncInfo(class_name, name, cpp_name, namespace, is_constructor)
+        func = func_map.setdefault(fi.name_id, fi)
 
         variant = FuncVariant(class_name, name, decl, is_constructor, is_class_method, is_const_method,
                         is_virtual_method, is_pure_virtual_method, ref_return, const_return)
@@ -429,7 +437,7 @@ class JSWrapperGenerator(object):
         f.write(buf.getvalue())
         f.close()
 
-    def gen_function_binding_with_wrapper(self, func, class_info):
+    def gen_function_binding_with_wrapper(self, func, ns_name, class_info):
 
         binding_text = None
         wrapper_func_text = None
@@ -462,8 +470,7 @@ class JSWrapperGenerator(object):
                     ret_type = type_dict[ptr_type]
             for key in type_dict:
                 if key in ret_type:
-                    ret_type = ret_type.replace(key, type_dict[key])
-
+                    ret_type = re.sub('(^|[^\w])' + key + '($|[^\w])', type_dict[key], ret_type)
             arg_types = []
             unwrapped_arg_types = []
             for arg in variant.args:
@@ -488,8 +495,23 @@ class JSWrapperGenerator(object):
 
 
             # Wrapper function
-            wrap_func_name = (func.class_name+"_" if class_info != None else "") + func.name.split("::")[-1] + "_wrapper"
-            js_func_name = func.name
+            if ns_name != None and ns_name != "cv":
+                ns_parts = ns_name.split(".")
+                if ns_parts[0] == "cv":
+                    ns_parts = ns_parts[1:]
+                ns_part = "_".join(ns_parts) + "_"
+                ns_id = '_'.join(ns_parts)
+                ns_prefix = namespace_prefix_override.get(ns_id, ns_id)
+                if ns_prefix:
+                    ns_prefix = ns_prefix + '_'
+            else:
+                ns_prefix = ''
+            if class_info == None:
+                js_func_name = ns_prefix + func.name
+                wrap_func_name = js_func_name + "_wrapper"
+            else:
+                wrap_func_name = ns_prefix + func.class_name + "_" + func.name + "_wrapper"
+                js_func_name = func.name
 
             # TODO: Name functions based wrap directives or based on arguments list
             if index > 0:
@@ -567,7 +589,7 @@ class JSWrapperGenerator(object):
                         # consider the default parameter variants
                         args_num = len(variant.args) - j
                         if args_num in class_info.constructor_arg_num:
-                            # FIXME: workaournd for constructor overload with same args number
+                            # FIXME: workaround for constructor overload with same args number
                             # e.g. DescriptorMatcher
                             continue
                         class_info.constructor_arg_num.add(args_num)
@@ -627,7 +649,6 @@ class JSWrapperGenerator(object):
             ret_type = 'void' if variant.rettype.strip() == '' else variant.rettype
 
             ret_type = ret_type.strip()
-
             if ret_type.startswith('Ptr'): #smart pointer
                 ptr_type = ret_type.replace('Ptr<', '').replace('>', '')
                 if ptr_type in type_dict:
@@ -741,12 +762,22 @@ class JSWrapperGenerator(object):
         # step 2: generate bindings
         # Global functions
         for ns_name, ns in sorted(self.namespaces.items()):
-            if ns_name.split('.')[0] != 'cv':
+            ns_parts = ns_name.split('.')
+            if ns_parts[0] != 'cv':
+                print('Ignore namespace: {}'.format(ns_name))
                 continue
-            for name, func in sorted(ns.funcs.items()):
+            else:
+                ns_parts = ns_parts[1:]
+            ns_id = '_'.join(ns_parts)
+            ns_prefix = namespace_prefix_override.get(ns_id, ns_id)
+            for name_id, func in sorted(ns.funcs.items()):
+                name = func.name
+                if ns_prefix:
+                    name = ns_prefix + '_' + name
                 if name in ignore_list:
                     continue
                 if not name in white_list['']:
+                    #print('Not in whitelist: "{}" from ns={}'.format(name, ns_name))
                     continue
 
                 ext_cnst = False
@@ -770,7 +801,7 @@ class JSWrapperGenerator(object):
                     continue
 
                 if with_wrapped_functions:
-                    binding, wrapper = self.gen_function_binding_with_wrapper(func, class_info=None)
+                    binding, wrapper = self.gen_function_binding_with_wrapper(func, ns_name, class_info=None)
                     self.bindings += binding
                     self.wrapper_funcs += wrapper
                 else:
@@ -803,7 +834,7 @@ class JSWrapperGenerator(object):
                         class_bindings.append(constructor_template.substitute(signature=', '.join(args)))
                 else:
                     if with_wrapped_functions and (len(method.variants) > 1 or len(method.variants[0].args)>0 or "String" in method.variants[0].rettype):
-                        binding, wrapper = self.gen_function_binding_with_wrapper(method, class_info=class_info)
+                        binding, wrapper = self.gen_function_binding_with_wrapper(method, None, class_info=class_info)
                         self.wrapper_funcs = self.wrapper_funcs + wrapper
                         class_bindings = class_bindings + binding
                     else:
