@@ -2,21 +2,38 @@
 // It is subject to the license terms in the LICENSE file found in the top-level directory
 // of this distribution and at http://opencv.org/license.html.
 
+//
+// DEPRECATED. Do not use in new plugins
+//
+
 #ifndef PLUGIN_API_HPP
 #define PLUGIN_API_HPP
 
 #include <opencv2/core/cvdef.h>
 #include <opencv2/core/llapi/llapi.h>
 
-// increase for backward-compatible changes, e.g. add new function
-// Main API <= Plugin API -> plugin is fully compatible
-// Main API > Plugin API -> plugin is not compatible, caller should use shim code to use plugins with old API
+#if !defined(BUILD_PLUGIN)
+
+/// increased for backward-compatible changes, e.g. add new function
+/// Caller API <= Plugin API -> plugin is fully compatible
+/// Caller API > Plugin API -> plugin is not fully compatible, caller should use extra checks to use plugins with older API
 #define API_VERSION 1 // preview
-// increase for incompatible changes, e.g. remove function argument
-// Main ABI == Plugin ABI -> plugin is compatible
-// Main ABI > Plugin ABI -> plugin is not compatible, caller should use shim code to use old ABI plugins
+
+/// increased for incompatible changes, e.g. remove function argument
+/// Caller ABI == Plugin ABI -> plugin is compatible
+/// Caller ABI > Plugin ABI -> plugin is not compatible, caller should use shim code to use old ABI plugins (caller may know how lower ABI works, so it is possible)
+/// Caller ABI < Plugin ABI -> plugin can't be used (plugin should provide interface with lower ABI to handle that)
 #define ABI_VERSION 0 // preview
 
+#else // !defined(BUILD_PLUGIN)
+
+#if !defined(ABI_VERSION) || !defined(API_VERSION)
+#error "Plugin must define ABI_VERSION and API_VERSION before including plugin_api.hpp"
+#endif
+
+#endif // !defined(BUILD_PLUGIN)
+
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -26,10 +43,8 @@ typedef CvResult (CV_API_CALL *cv_videoio_retrieve_cb_t)(int stream_idx, unsigne
 typedef struct CvPluginCapture_t* CvPluginCapture;
 typedef struct CvPluginWriter_t* CvPluginWriter;
 
-typedef struct OpenCV_VideoIO_Plugin_API_preview
+struct OpenCV_VideoIO_Plugin_API_v0_0_api_entries
 {
-    OpenCV_API_Header api_header;
-
     /** OpenCV capture ID (VideoCaptureAPIs)
     @note API-ENTRY 1, API-Version == 0
      */
@@ -118,7 +133,7 @@ typedef struct OpenCV_VideoIO_Plugin_API_preview
 
     /** @brief Get property value
 
-    @param handle Capture handle
+    @param handle Writer handle
     @param prop Property index
     @param[out] val property value
 
@@ -128,7 +143,7 @@ typedef struct OpenCV_VideoIO_Plugin_API_preview
 
     /** @brief Set property value
 
-    @param handle Capture handle
+    @param handle Writer handle
     @param prop Property index
     @param val property value
 
@@ -138,8 +153,8 @@ typedef struct OpenCV_VideoIO_Plugin_API_preview
 
     /** @brief Write frame
 
-    @param handle Capture handle
-    @param data Capture handle
+    @param handle Writer handle
+    @param data frame data
     @param step step in bytes
     @param width frame width in pixels
     @param height frame height
@@ -148,8 +163,10 @@ typedef struct OpenCV_VideoIO_Plugin_API_preview
     @note API-CALL 12, API-Version == 0
      */
     CvResult (CV_API_CALL *Writer_write)(CvPluginWriter handle, const unsigned char *data, int step, int width, int height, int cn);
+}; // OpenCV_VideoIO_Plugin_API_v0_0_api_entries
 
-
+struct OpenCV_VideoIO_Plugin_API_v0_1_api_entries
+{
     /** @brief Try to open video writer
 
     @param filename Destination location
@@ -168,19 +185,32 @@ typedef struct OpenCV_VideoIO_Plugin_API_preview
         int* params, unsigned n_params,
         CV_OUT CvPluginWriter* handle
     );
+}; // OpenCV_VideoIO_Plugin_API_v0_1_api_entries
 
-} OpenCV_VideoIO_Plugin_API_preview;
+typedef struct OpenCV_VideoIO_Plugin_API_preview_v0
+{
+    OpenCV_API_Header api_header;
+    struct OpenCV_VideoIO_Plugin_API_v0_0_api_entries v0;
+} OpenCV_VideoIO_Plugin_API_preview_v0;
+
+typedef struct OpenCV_VideoIO_Plugin_API_preview_v1
+{
+    OpenCV_API_Header api_header;
+    struct OpenCV_VideoIO_Plugin_API_v0_0_api_entries v0;
+    struct OpenCV_VideoIO_Plugin_API_v0_1_api_entries v1;
+} OpenCV_VideoIO_Plugin_API_preview_v1;
+
+
+#if ABI_VERSION == 0 && API_VERSION == 1
+typedef struct OpenCV_VideoIO_Plugin_API_preview_v1 OpenCV_VideoIO_Plugin_API_preview;
+#elif ABI_VERSION == 0 && API_VERSION == 0
+typedef struct OpenCV_VideoIO_Plugin_API_preview_v0 OpenCV_VideoIO_Plugin_API_preview;
+#else
+#error "Not supported configuration: check ABI_VERSION/API_VERSION"
+#endif
 
 #ifdef BUILD_PLUGIN
 
-#ifndef CV_PLUGIN_EXPORTS
-#if (defined _WIN32 || defined WINCE || defined __CYGWIN__)
-#  define CV_PLUGIN_EXPORTS __declspec(dllexport)
-#elif defined __GNUC__ && __GNUC__ >= 4
-#  define CV_PLUGIN_EXPORTS __attribute__ ((visibility ("default")))
-#endif
-#endif
-
 CV_PLUGIN_EXPORTS
 const OpenCV_VideoIO_Plugin_API_preview* CV_API_CALL opencv_videoio_plugin_init_v0
         (int requested_abi_version, int requested_api_version, void* reserved /*NULL*/) CV_NOEXCEPT;
