@@ -189,7 +189,10 @@ class BuilderDLDT:
         if self.srcdir is None:
             self.srcdir = prepare_dir(self.outdir / 'sources', clean=clean_src_dir)
         self.build_dir = prepare_dir(self.outdir / 'build', clean=self.config.clean_dldt)
-        self.sysrootdir = prepare_dir(self.outdir / 'sysroot', clean=self.config.clean_dldt)
+        self.sysrootdir = prepare_dir(self.outdir / 'sysroot', clean=self.config.clean_dldt or self.config.clean_dldt_sysroot)
+        if not (self.config.clean_dldt or self.config.clean_dldt_sysroot):
+            _ = prepare_dir(self.sysrootdir / 'bin', clean=True)  # always clean sysroot/bin (package files)
+            _ = prepare_dir(self.sysrootdir / 'etc', clean=True)  # always clean sysroot/etc (package files)
 
         if self.config.build_subst_drive:
             if os.path.exists(self.config.build_subst_drive + ':\\'):
@@ -214,7 +217,7 @@ class BuilderDLDT:
             patch_hashsum = hashlib.md5(self.patch_file_contents.encode('utf-8')).hexdigest()
         except:
             log.warn("Can't compute hashsum of patches: %s", self.patch_file)
-        self.patch_hashsum = patch_hashsum
+        self.patch_hashsum = self.config.override_patch_hashsum if self.config.override_patch_hashsum else patch_hashsum
 
 
     def prepare_sources(self):
@@ -300,7 +303,9 @@ class BuilderDLDT:
 
             # build
             cmd = [self.cmake_path, '--build', '.', '--config', build_config, # '--target', 'install',
-                    '--', '/v:n', '/m:2', '/consoleloggerparameters:NoSummary'
+                    '--',
+                    # '/m:2' is removed, not properly supported by 2021.3
+                    '/v:n', '/consoleloggerparameters:NoSummary',
             ]
             execute(cmd, cwd=build_dir)
 
@@ -353,7 +358,6 @@ class Builder:
             BUILD_PERF_TESTS='OFF',
             ENABLE_CXX11='ON',
             WITH_INF_ENGINE='ON',
-            INF_ENGINE_RELEASE=str(self.config.dldt_release),
             WITH_TBB='ON',
             CPU_BASELINE='AVX2',
             CMAKE_INSTALL_PREFIX=str(self.install_dir),
@@ -381,10 +385,22 @@ class Builder:
             OPENCV_PYTHON_INSTALL_PATH='python',
         )
 
+        if self.config.dldt_release:
+            cmake_vars['INF_ENGINE_RELEASE'] = str(self.config.dldt_release)
+
         cmake_vars['INF_ENGINE_LIB_DIRS:PATH'] = str(builderDLDT.sysrootdir / 'deployment_tools/inference_engine/lib/intel64')
+        assert os.path.exists(cmake_vars['INF_ENGINE_LIB_DIRS:PATH']), cmake_vars['INF_ENGINE_LIB_DIRS:PATH']
         cmake_vars['INF_ENGINE_INCLUDE_DIRS:PATH'] = str(builderDLDT.sysrootdir / 'deployment_tools/inference_engine/include')
-        cmake_vars['ngraph_DIR:PATH'] = str(builderDLDT.sysrootdir / 'ngraph/cmake')
+        assert os.path.exists(cmake_vars['INF_ENGINE_INCLUDE_DIRS:PATH']), cmake_vars['INF_ENGINE_INCLUDE_DIRS:PATH']
+
+        ngraph_DIR = str(builderDLDT.sysrootdir / 'ngraph/cmake')
+        if not os.path.exists(ngraph_DIR):
+            ngraph_DIR = str(builderDLDT.sysrootdir / 'ngraph/deployment_tools/ngraph/cmake')
+        assert os.path.exists(ngraph_DIR), ngraph_DIR
+        cmake_vars['ngraph_DIR:PATH'] = ngraph_DIR
+
         cmake_vars['TBB_DIR:PATH'] = str(builderDLDT.sysrootdir / 'tbb/cmake')
+        assert os.path.exists(cmake_vars['TBB_DIR:PATH']), cmake_vars['TBB_DIR:PATH']
 
         if self.config.build_debug:
             cmake_vars['CMAKE_BUILD_TYPE'] = 'Debug'
@@ -455,8 +471,9 @@ class Builder:
 def main():
 
     dldt_src_url = 'https://github.com/openvinotoolkit/openvino'
-    dldt_src_commit = '2021.2'
-    dldt_release = '2021020000'
+    dldt_src_commit = '2021.4.1'
+    dldt_config = None
+    dldt_release = None
 
     build_cache_dir_default = os.environ.get('BUILD_CACHE_DIR', '.build_cache')
     build_subst_drive = os.environ.get('BUILD_SUBST_DRIVE', None)
@@ -472,8 +489,9 @@ def main():
     parser.add_argument('--cmake_option', action='append', help='Append OpenCV CMake option')
     parser.add_argument('--cmake_option_dldt', action='append', help='Append CMake option for DLDT project')
 
-    parser.add_argument('--clean_dldt', action='store_true', help='Clear DLDT build and sysroot directories')
-    parser.add_argument('--clean_opencv', action='store_true', help='Clear OpenCV build directory')
+    parser.add_argument('--clean_dldt', action='store_true', help='Clean DLDT build and sysroot directories')
+    parser.add_argument('--clean_dldt_sysroot', action='store_true', help='Clean DLDT sysroot directories')
+    parser.add_argument('--clean_opencv', action='store_true', help='Clean OpenCV build directory')
 
     parser.add_argument('--build_debug', action='store_true', help='Build debug binaries')
     parser.add_argument('--build_tests', action='store_true', help='Build OpenCV tests')
@@ -483,12 +501,14 @@ def main():
     parser.add_argument('--dldt_src_branch', help='DLDT checkout branch')
     parser.add_argument('--dldt_src_commit', default=dldt_src_commit, help='DLDT source commit / tag (default: %s)' % dldt_src_commit)
     parser.add_argument('--dldt_src_git_clone_extra', action='append', help='DLDT git clone extra args')
-    parser.add_argument('--dldt_release', default=dldt_release, help='DLDT release code for INF_ENGINE_RELEASE (default: %s)' % dldt_release)
+    parser.add_argument('--dldt_release', default=dldt_release, help='DLDT release code for INF_ENGINE_RELEASE, e.g 2021030000 (default: %s)' % dldt_release)
 
     parser.add_argument('--dldt_reference_dir', help='DLDT reference git repository (optional)')
     parser.add_argument('--dldt_src_dir', help='DLDT custom source repository (skip git checkout and patching, use for TESTING only)')
 
-    parser.add_argument('--dldt_config', help='Specify DLDT build configuration (defaults to evaluate from DLDT commit/branch)')
+    parser.add_argument('--dldt_config', default=dldt_config, help='Specify DLDT build configuration (defaults to evaluate from DLDT commit/branch)')
+
+    parser.add_argument('--override_patch_hashsum', default='', help='(script debug mode)')
 
     args = parser.parse_args()
 
@@ -514,8 +534,12 @@ def main():
         args.opencv_dir = os.path.abspath(args.opencv_dir)
 
     if not args.dldt_config:
-        if args.dldt_src_commit == 'releases/2020/4' or args.dldt_src_branch == 'releases/2020/4':
-            args.dldt_config = '2020.4'
+        if str(args.dldt_src_commit).startswith('releases/20'):  # releases/2020/4
+            args.dldt_config = str(args.dldt_src_commit)[len('releases/'):].replace('/', '.')
+            if not args.dldt_src_branch:
+                args.dldt_src_branch = args.dldt_src_commit
+        elif str(args.dldt_src_branch).startswith('releases/20'):  # releases/2020/4
+            args.dldt_config = str(args.dldt_src_branch)[len('releases/'):].replace('/', '.')
         else:
             args.dldt_config = args.dldt_src_commit
 
